#include <stdio.h>
#include <unistd.h>

#include "main.h"
#include "usb.h"
#include "atb-usbasp.h"


struct TusbControlPacket controlPacket;
char buffer[ATB_USPASP_MAX_BUFFER_SIZE];


char scanI2C ( void )
{
	unsigned char i;
	unsigned char count = 0;

	printf("Searching I2C devices ...\n");
	fflush(stdout);
	for (i = I2C_7BIT_ADDRESS_MIN; i <= I2C_7BIT_ADDRESS_MAX; i++)
	{
		switch (checkI2CDevice(i << 1))
		{
			case ERROR:
				printf("Communication with ATB-USBasp failed.\n");
				return ERROR;
			case SUCCESS:
				count++;
				printf("\t%2d. Found I2C device: \t7-bit address: 0x%02X (%d)	 \t8-bit address: 0x%02X (%d)\n", count, i, i, i << 1, i << 1);
				break;
		}
		if (receivedTermSignal) return ERROR;
	}
	if (count == 0) printf("\tNo I2C devices found\n");
	return SUCCESS;
}


char setI2CBitrate ( unsigned short bitrate )
{
		controlPacket.bmRequestType = 0xC0;						// read from USB device
		controlPacket.bRequest = USBASP_FUNC_I2C_BITRATE;		// ATB-USBasp API command
		controlPacket.wValue = bitrate;							// I2C bitrate
		controlPacket.wIndex = 0;								// not used
		controlPacket.data = buffer;							// buffer
		controlPacket.wLength = ATB_USPASP_MAX_BUFFER_SIZE;		// buffer size

		return usbSendControlData(controlPacket);
}


char checkI2CDevice ( unsigned char address )
{
	controlPacket.bmRequestType = 0xC0;							// read from USB device
	controlPacket.bRequest = USBASP_I2C_SLA_CHECK;				// ATB-USBasp API command
	controlPacket.wValue = address;								// I2C address
	controlPacket.wIndex = 0;									// not used
	controlPacket.data = buffer;								// buffer
	controlPacket.wLength = ATB_USPASP_MAX_BUFFER_SIZE;			// buffer size

	if (usbSendControlData(controlPacket) != SUCCESS)
	{
		return ERROR;
	}
	else
	{
		if ((controlPacket.data[0] == 'O') && (controlPacket.data[1] == 'K')) return SUCCESS;
		else return DEVICE_NO_FOUND;
	}
}


char toggleI2CPins ( void )
{
	char state = 1;

	controlPacket.bmRequestType = 0x40;							// write to USB device
	controlPacket.bRequest = USBASP_I2C_BITBANG_ON;				// ATB-USBasp API command
	controlPacket.wValue = 0;									// not used
	controlPacket.wIndex = 0;									// not used
	controlPacket.wLength = 0;									// not used
	controlPacket.data = buffer;								// buffer
	controlPacket.wLength = ATB_USPASP_MAX_BUFFER_SIZE;			// buffer size

	if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;					// send Control Packet
	printf("Toggling pins ...  - -");
	fflush(stdout);
	while (1)
	{
		switch (state)
		{
			case 1:
					controlPacket.bRequest = USBASP_I2C_SDA_LO;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					controlPacket.bRequest = USBASP_I2C_SCL_HI;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					break;

			case 2:
					controlPacket.bRequest = USBASP_I2C_SDA_HI;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					controlPacket.bRequest = USBASP_I2C_SCL_LO;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					break;

			case 3:
					controlPacket.bRequest = USBASP_I2C_SDA_HI;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					controlPacket.bRequest = USBASP_I2C_SCL_HI;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					break;

			default:
					controlPacket.bRequest = USBASP_I2C_SDA_LO;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					controlPacket.bRequest = USBASP_I2C_SCL_LO;						// new ATB-USBasp API command
					if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;	// send Control Packet
					state = 0;
		}
		printf("\b\b\b%c %c", state & 0x01 ? '1' : '0', state & 0x02 ? '1' : '0');
		fflush(stdout);
		state++;
		usleep(500000);
		if (receivedTermSignal) break;												// break when user request
	}
	controlPacket.bRequest = USBASP_I2C_BITBANG_OFF;								// new ATB-USBasp API command
	if (usbSendControlData(controlPacket) != SUCCESS) return ERROR;					// send Control Packet
	return SUCCESS;
}
