#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#ifndef WIN32
	#include <termios.h>
#endif

#include "main.h"
#include "atb-usbasp.h"
#include "usb.h"

volatile char receivedTermSignal = 0;
volatile char HUPsignal = 1;

void termSignalApplication(int sig);


int main(int argc, char *argv[])
{
#ifdef TOGGLE_I2C_PINS
	printf(APPLICATION_NAME_TOGGLE" ver. "APPLICATION_VERSION"\n");
#else
	printf(APPLICATION_NAME" ver. "APPLICATION_VERSION"\n");
#endif

#ifndef WIN32
	struct sigaction termSignalAction;

	memset(&termSignalAction, 0, sizeof(termSignalAction));
	termSignalAction.sa_handler = &termSignalApplication;
	if (sigaction(SIGTERM, &termSignalAction, NULL) < 0)
	{
		printf("Creating SIGTERM failed. Application terminated.\n");
		return ERROR;
	}
	if (sigaction(SIGINT, &termSignalAction, NULL) < 0)
	{
		printf("Creating SIGINT failed. Application terminated.\n");
		return ERROR;
	}
#endif


	if (openUSB() == ERROR)
	{
		printf("Can't open ATB-USBasp device.\n");
		return ERROR;
	}

#ifdef TOGGLE_I2C_PINS

	if (toggleI2CPins() != SUCCESS) printf("Communication with ATB-USBasp failed.\n");

#else

	if (argc > 1)
	{
		unsigned short bitrate = atoi(argv[1]);
		if ((bitrate < USBASP_I2C_MIN_BITRATE) || (bitrate > USBASP_I2C_MAX_BITRATE))
		{
			printf("Incorrect I2C bitrate value. Using default.\n");
		}
		else
		{
			if (setI2CBitrate(bitrate) == SUCCESS)
			{
			 	printf("I2C bitrate set to %d kbps.\n", bitrate);				
			}
			else
			{
			 	printf("Setting I2C bitrate failed.\n");
			}
		}
	}
	scanI2C();

#endif


	closeUSB();

	return SUCCESS;
}


void termSignalApplication(int sig)
{
	receivedTermSignal = 1;
	switch(sig)
	{
		case SIGINT:
			printf("\nReceived signal SIGINT, closing ...\n");
			break;
		case SIGTERM:
			printf("\nReceived signal SIGTERM, closing ...\n");
			break;
	}
}

